/* Handles events from SDL.
   Copyright (C) 2006 Carl Fredrik Hammar (hammy.lite@gmail.com)

   This file is part of GP2X Sand.

   GP2X Sand is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   GP2X Sand is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details. 

   You should have received a copy of the GNU General Public License along
   with GP2X Sand; if not, write to the Free Software Foundation, Inc., 51
   Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA */

#ifndef EVENTHANDLER_H
#define EVENTHANDLER_H

#include "SDL.h"
#include "action.h"

/* The event-handler maps events generated by the keyboard and joystick to
   actions and invokes the actions when the corresponding event occurs. */

/* Initialize the event-handler, with no mappings. */
void
eventhandler_init_default ();

/* Process the events in SDL's event queue and invoke corresponding
   actions.  Call this regularly. */
void
eventhandler_process_events ();

/* Definition of a single mapping from a button (either a SDLKey or
   joystick button number) to an action.  Mostly used for convenient*. */
typedef struct { Uint16 button; action_t *action; }
mapping_button_action_t;

/* Map the SDL_Quit event to the action pointed to by ACTION_PTR. */
void
eventhandler_quit_map (action_t *action_ptr);

/* Map event generated by KEY to the action pointed to by ACTION_PTR. 
   KEY must be a valid keyboard key, */
void
eventhandler_key_map (SDLKey key, action_t *action_ptr);

/* Map all the mappings specified by the mappings in the array MAP, where
   all button entries will be viewed as keyboard keys. This is equivalent
   to calling eventhandler_key_map for each entry in MAP.  MAP_LENGTH must
   be the length of MAP. */
void
eventhandler_key_map_load (mapping_button_action_t *map,
			   size_t map_lenght)
  __attribute__ ((nonnull (1)));

/* Map event generated by joystick button JOYBUTTON to the action pointed
   to by ACTION_PTR. JOYBUTTON must be between 0 and 255. */ 
void
eventhandler_joybutton_map (int joybutton, action_t *action);
  
/* Map all the mappings specified by the mappings in the array MAP, where
   all button entries will be viewed as joystick buttons. This is
   equivalent to calling eventhandler_joybutton_map for each entry in MAP.
   MAP_LENGTH must be the length of MAP. */
void
eventhandler_joybutton_map_load (mapping_button_action_t *map,
				 size_t map_lenght)
  __attribute__ ((nonnull (1)));

#endif /* not EVENTHANDLER_H */
