/* Map keyboard keys to emulate a GP2X's buttons.
   Copyright (C) 2006 Carl Fredrik Hammar (hammy.lite@gmail.com)

   This file is part of GP2X Sand.

   GP2X Sand is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   GP2X Sand is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details. 

   You should have received a copy of the GNU General Public License along
   with GP2X Sand; if not, write to the Free Software Foundation, Inc., 51
   Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA */

#include "gp2xkeyboard.h"

#include "SDL.h"
#include <assert.h>

#include "action.h"
#include "gp2xbuttons.h"
#include "eventhandler.h"
#include "utils.h"

struct gp2x_keyboard_action
{
  action_t action;
  Uint8 joybutton;
};
  
typedef struct gp2x_keyboard_action gp2x_keyboard_action_t;

gp2x_keyboard_action_t map_action[GP2X_BUTTONS_N];

void
gp2x_keyboard_action_init (gp2x_keyboard_action_t *action_ptr,
			   Uint8 joybutton);

void
gp2x_keyboard_action_function (void *action_ptr, SDL_Event *event_ptr);

void
gp2x_keyboard_init_default ()
{
  int map_key[] = { [GP2X_BUTTON_UP]        = SDLK_KP8,      
		    [GP2X_BUTTON_DOWN]      = SDLK_KP2,
		    [GP2X_BUTTON_LEFT]      = SDLK_KP4,
		    [GP2X_BUTTON_RIGHT]     = SDLK_KP6,
		    [GP2X_BUTTON_UPLEFT]    = SDLK_KP7,
		    [GP2X_BUTTON_UPRIGHT]   = SDLK_KP9,
		    [GP2X_BUTTON_DOWNLEFT]  = SDLK_KP1,
		    [GP2X_BUTTON_DOWNRIGHT] = SDLK_KP3,
		    [GP2X_BUTTON_CLICK]     = SDLK_KP5,
		    
		    [GP2X_BUTTON_A]         = SDLK_a,
		    [GP2X_BUTTON_B]         = SDLK_d,
		    [GP2X_BUTTON_X]         = SDLK_s,
		    [GP2X_BUTTON_Y]         = SDLK_w,
		    
		    [GP2X_BUTTON_L]         = SDLK_q,
		    [GP2X_BUTTON_R]         = SDLK_e,
		    
		    [GP2X_BUTTON_START]     = SDLK_RETURN,
		    [GP2X_BUTTON_SELECT]    = SDLK_SPACE,
		    
		    [GP2X_BUTTON_VOLUP]     = SDLK_KP_PLUS,
		    [GP2X_BUTTON_VOLDOWN]   = SDLK_KP_MINUS };
  
  for (int joybutton = 0; joybutton < LENGTH_OF (map_key); joybutton++)
    {
      gp2x_keyboard_action_init (&map_action[joybutton], joybutton);
      eventhandler_key_map (map_key[joybutton],
			    (action_t*) &map_action[joybutton]);
    }
}

void
gp2x_keyboard_action_init (gp2x_keyboard_action_t *action_ptr,
			   Uint8 joybutton)
{
  assert (action_ptr != 0);

  action_init ((action_t*) action_ptr, gp2x_keyboard_action_function);
  action_ptr->joybutton = joybutton;
}

void
gp2x_keyboard_action_function (void *action_ptr, SDL_Event *event_ptr)
{
  assert (action_ptr != 0);
  assert (event_ptr != 0);
  
  gp2x_keyboard_action_t *gp2x_action_ptr = action_ptr;
  Uint8 joybutton = gp2x_action_ptr->joybutton;
  
  if (event_trigger_state (event_ptr) == TRIGGER_DOWN)
    {
      SDL_JoyButtonEvent event = { SDL_JOYBUTTONDOWN, 0, joybutton,
				   SDL_PRESSED };
      SDL_PushEvent ((SDL_Event*) &event);
    }
  
  else if (event_trigger_state (event_ptr) == TRIGGER_UP)
    {
      SDL_JoyButtonEvent event = { SDL_JOYBUTTONUP, 0, joybutton,
				   SDL_RELEASED };
      SDL_PushEvent ((SDL_Event*) &event);
    }
}
